==> kxdocker_0.26.orig.tar.gz: kxdocker-0.26.orig/src/xeplugin_mouse.cpp <==
/***************************************************************************
 *   Copyright (C) 2004 by Stefano                                         *
 *   stefano@xiaprojects.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "xeplugin_mouse.h"
#include <xeobject.h>
#include <xeconfiguration.h>
#include <xgdocker.h>
#include <qevent.h> 
#include <kapplication.h>
//#include <kmainwindow.h>
#include <stdlib.h>

#include <kxdocker_setup_sources.h>
//#undef ENABLE_FINAL
XEPlugin_Mouse::XEPlugin_Mouse(QObject *parent, const char *name)
 : QObject(parent, name)
{
	XEObject::xPluginAdd(this);
	display = kapp->getDisplay();
	root = RootWindow(display,DefaultScreen(display));
}


XEPlugin_Mouse::~XEPlugin_Mouse()
{
}


#include "xeplugin_mouse.moc"


/*!
    \fn XEPlugin_Mouse::xSetup()
 */
void XEPlugin_Mouse::xSetup()
{
#ifndef ENABLE_FINAL
	qWarning("Mouse::xSetup");
#endif
	XEConfiguration *Configurator=(XEConfiguration *)XEObject::xGetConfiguration();
	xGDocker=(XGDocker *)XEObject::xGetDocker();
	if(Configurator==NULL || xGDocker==NULL)
	{
#ifndef ENABLE_FINAL
	// Emits Signal of warning
	//xWarningMsg(this,"error while searching for plugin");
	qWarning("error while searching for plugin");
#endif
		return;
	}
	ActiveConfiguration=Configurator->xGetConfiguration();

}


/*!
    \fn XEPlugin_Mouse::xStart()
 */
void XEPlugin_Mouse::xStart()
{
#ifndef ENABLE_FINAL
	qWarning("Mouse::xStart");
#endif
	connect(xGDocker,SIGNAL(xEventMouseMoved(int,int)),
		this,SLOT(xEventMouseMoved(int,int)));
	
	connect(xGDocker,SIGNAL(xEventDockerHidden()),
		this,SLOT(xEventDockerHidden()));
	
	connect(this,SIGNAL(xGetWidget(QWidget*& )),
		xGDocker,SLOT(xGetWidget(QWidget*& )));
	
	connect(this,SIGNAL(mouseMoveSoftware(int, int )),
		xGDocker,SLOT(xMouseMoveSoftware(int, int )));
	xGetWidget(xGDockerWindow);
	
}


/*!
    \fn XEPlugin_Mouse::xStop()
 */
void XEPlugin_Mouse::xStop()
{
    /// @todo implement me
}


/*!
    \fn XEPlugin_Mouse::xEventMouseMoved(int x, int y, int button)
 */
void XEPlugin_Mouse::xEventMouseMoved(int nx,int ny)
{
//	MouseTimer.singleShot(10, this, SLOT(stepMouseTimer()));
	x=nx;
	y=ny;
#ifndef ENABLE_FINAL
	qWarning("MouseTracking");
#endif
	stepMouseTimer();
}


/*!
    \fn XEPlugin_Mouse::stepMouseTimer()
 */
void XEPlugin_Mouse::stepMouseTimer()
{
	if(MousePollingMutex.tryLock()==false)return;
	QPoint pos, mouse, vect;
	int WinX, WinY;
	int XCoord,YCoord;
	XQueryPointer(display, root, &RootIDRet, &ChildIDRet, &XCoord,&YCoord, &WinX, &WinY, &StateMask);
	//mouse = xGDocker->mapFromGlobal(QPoint(XCoord,YCoord));
	mouse = xGDockerWindow->mapFromGlobal(QPoint(XCoord,YCoord));
	//if(mouse.x()==x||mouse.y()==y)return;
	if(mouse.y()>=0)
	{
		MousePollingMutex.unlock();
		return;
	}
	mouseMoveSoftware(mouse.x(),mouse.y());
//	xGDocker->mouseMoveSoftware(mouse.x(),mouse.y());
#ifndef ENABLE_FINAL
	qWarning("MouseTracking:End");
#endif
	MousePollingMutex.unlock();
	
}


/*!
    \fn XEPlugin_Mouse::xEventDockerHidden()
 */
void XEPlugin_Mouse::xEventDockerHidden()
{
#ifndef ENABLE_FINAL
	qWarning("XEPlugin_Mouse::xEventDockerHidden()");
#endif
	stepMouseTracking();
}


/*!
    \fn XEPlugin_Mouse::stepMouseTracking()
 */
void XEPlugin_Mouse::stepMouseTracking()
{
#ifndef ENABLE_FINAL
	qWarning("XEPlugin_Mouse::stepMouseTracking()");
#endif
	
	int PollingTimeout=200;
	static int CountStep=0;
	static int PollingConversion=ActiveConfiguration->Window.SendToForgroundTimeout/PollingTimeout;

	// 0.22 support send to background which is not compatible with this!
	//if(xGDocker->getRaised()>0 || !xGDocker->isHidden())return;
	if(xGDocker->getRaised()>0 )return;
	
	if(MousePollingMutex.tryLock()==false)return;
	
	QPoint pos, mouse, vect;
	int WinX, WinY;
	int XCoord,YCoord;
	XQueryPointer(display, root, &RootIDRet, &ChildIDRet, &XCoord,&YCoord, &WinX, &WinY, &StateMask);
	
	mouse = xGDocker->mapFromGlobal(QPoint(XCoord,YCoord));
	
	if(strcmp(ActiveConfiguration->Window.Align,"bottom")==0)
	{
		// idea from georg
		// raise the bar only if there are 2 corner
		int MouseIsOk=0;
		
		if(ActiveConfiguration->Window.HideMouseCornerRight)
		{
		if(XCoord>=
		QApplication::desktop()->width()-
		ActiveConfiguration->Window.HideMouseEdge
		)
		{
			MouseIsOk++;
		}
		else MouseIsOk--;

		}
		if(ActiveConfiguration->Window.HideMouseCornerLeft)
		{
		if(XCoord<=
		ActiveConfiguration->Window.HideMouseEdge
		)
		{
			MouseIsOk++;
		}
		else MouseIsOk--;
		}
		if(YCoord>=
		QApplication::desktop()->height()-
		ActiveConfiguration->Window.HideMouseEdge
		)
		{
			MouseIsOk++;

		}
		else MouseIsOk--;

		if(MouseIsOk>0)
		{
				
			if(CountStep<PollingConversion)CountStep++;
			else
			{
			CountStep=0;
			// ok than we have to raise the widget!
			
			xGDocker->showRaised();
#ifdef ENABLE_FINAL
			qWarning("MouseTracking:End RAISED!!!");
#endif
			MousePollingMutex.unlock();
			return;
			}

		}
		else CountStep=0;

	}
	if(strcmp(ActiveConfiguration->Window.Align,"top")==0)
	{
		// idea from georg
		// raise the bar only if there are 2 corner
		int MouseIsOk=0;
		
		if(ActiveConfiguration->Window.HideMouseCornerRight)
		{
		if(XCoord>=
		QApplication::desktop()->width()-
		ActiveConfiguration->Window.HideMouseEdge
		)
		{
			MouseIsOk++;
		}
		else MouseIsOk--;
		
		}
		if(ActiveConfiguration->Window.HideMouseCornerLeft)
		{
		if(XCoord<=
		ActiveConfiguration->Window.HideMouseEdge
		)
		{
			MouseIsOk++;
		}
		else MouseIsOk--;
		
		}

		if(YCoord<=ActiveConfiguration->Window.HideMouseEdge
		)
		{
			MouseIsOk++;
		}
		else MouseIsOk--;

		if(MouseIsOk>0)
		{
			if(CountStep<PollingConversion)CountStep++;
			else
			{
			CountStep=0;
			xGDocker->showRaised();
#ifdef ENABLE_FINAL
			qWarning("MouseTracking:End RAISED!!!");
#endif
			MousePollingMutex.unlock();
			return;
			}
		}
		else CountStep=0;

	}
#ifndef ENABLE_FINAL
	qWarning("MouseTracking:End");
#endif
	MousePollingMutex.unlock();
	
	QTimer::singleShot(PollingTimeout,this,SLOT(stepMouseTracking()));
}


/*!
    \fn XEPlugin_Mouse::xGetParameter(const QString,QString&)
 */
void XEPlugin_Mouse::xGetParameter(const QString,QString&)
{
    /// @todo implement me
}


/*!
    \fn XEPlugin_Mouse::xGetParameterList(QStringList*)
 */
void XEPlugin_Mouse::xGetParameterList(QStringList*)
{
    /// @todo implement me
}

==> kxdocker_0.39.orig.tar.gz: kxdocker-0.39/src/xeplugin_mouse.cpp <==
/***************************************************************************
 *   Copyright (C) 2004 by Stefano                                         *
 *   stefano@xiaprojects.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "xeplugin_mouse.h"
#include <xeobject.h>
#include <xeconfiguration.h>
#include <xgdocker.h>
#include <qevent.h> 
#include <qtimer.h>

#include <kapplication.h>
//#include <kmainwindow.h>
#include <stdlib.h>

#include <kxdocker_setup_sources.h>
//#undef ENABLE_FINAL
XEPlugin_Mouse::XEPlugin_Mouse(QObject *parent, const char *name)
 : QObject(parent, name)
{
	XEObject::xPluginAdd(this);
	display = kapp->getDisplay();
	root = RootWindow(display,DefaultScreen(display));
	MouseTimer=new QTimer();
	PollingTimeout=100;
}


XEPlugin_Mouse::~XEPlugin_Mouse()
{
	//qWarning("XEPlugin_Mouse::~XEPlugin_Mouse()");
}


#include "xeplugin_mouse.moc"


/*!
    \fn XEPlugin_Mouse::xSetup()
 */
void XEPlugin_Mouse::xSetup()
{
#ifndef ENABLE_FINAL
	qWarning("Mouse::xSetup");
#endif
	XEConfiguration *Configurator=(XEConfiguration *)XEObject::xGetConfiguration();
	xGDocker=(XGDocker *)XEObject::xGetDocker();
	if(Configurator==NULL || xGDocker==NULL)
	{
#ifndef ENABLE_FINAL
	// Emits Signal of warning
	//xWarningMsg(this,"error while searching for plugin");
	qWarning("error while searching for plugin");
#endif
		return;
	}
	ActiveConfiguration=Configurator->xGetConfiguration();

}


/*!
    \fn XEPlugin_Mouse::xStart()
 */
void XEPlugin_Mouse::xStart()
{
#ifndef ENABLE_FINAL
	qWarning("Mouse::xStart");
#endif
	connect(xGDocker,SIGNAL(xEventMouseMoved(int,int)),
		this,SLOT(xEventMouseMoved(int,int)));
	
	connect(xGDocker,SIGNAL(xEventDockerHidden()),
		this,SLOT(xEventDockerHidden()));
	
	connect(xGDocker,SIGNAL(xEventDockerSendToBackground()),this,SLOT(xEventDockerSendToBackground()));
	connect(MouseTimer,SIGNAL(timeout()),this,SLOT(stepMouseTracking()));
	
	connect(this,SIGNAL(xGetWidget(QWidget*& )),
		xGDocker,SLOT(xGetWidget(QWidget*& )));
	
	connect(this,SIGNAL(mouseMoveSoftware(int, int )),
		xGDocker,SLOT(xMouseMoveSoftware(int, int )));
	xGetWidget(xGDockerWindow);
	
}


/*!
    \fn XEPlugin_Mouse::xStop()
 */
void XEPlugin_Mouse::xStop()
{
	//qWarning("void XEPlugin_Mouse::xStop()");
	MousePollingMutex.tryLock();
}


/*!
    \fn XEPlugin_Mouse::xEventMouseMoved(int x, int y, int button)
 */
void XEPlugin_Mouse::xEventMouseMoved(int nx,int ny)
{
	x=nx;
	y=ny;
#ifndef ENABLE_FINAL
	qWarning("MouseTracking");
#endif
	stepMouseTimer();
	
}


/*!
    \fn XEPlugin_Mouse::stepMouseTimer()
 */
void XEPlugin_Mouse::stepMouseTimer()
{
	if(MousePollingMutex.tryLock()==false)return;
	QPoint pos, mouse, vect;
	int WinX, WinY;
	int XCoord,YCoord;
	XQueryPointer(display, root, &RootIDRet, &ChildIDRet, &XCoord,&YCoord, &WinX, &WinY, &StateMask);
	//mouse = xGDocker->mapFromGlobal(QPoint(XCoord,YCoord));
	mouse = xGDockerWindow->mapFromGlobal(QPoint(XCoord,YCoord));
	//if(mouse.x()==x||mouse.y()==y)return;
	if(mouse.y()>=0)
	{
		MousePollingMutex.unlock();
		return;
	}
	mouseMoveSoftware(mouse.x(),mouse.y());
//	xGDocker->mouseMoveSoftware(mouse.x(),mouse.y());
#ifndef ENABLE_FINAL
	qWarning("MouseTracking:End");
#endif
	MousePollingMutex.unlock();
	
}

/*!
    \fn XEPlugin_Mouse::xEventDockerSendToBackground()
 */
void XEPlugin_Mouse::xEventDockerSendToBackground()
{
#ifndef ENABLE_FINAL
	qWarning("XEPlugin_Mouse::xEventDockerHidden()");
#endif
	// TODO: we have to do only ontop :)
	// but we have to manage a better mutex
	if(!MouseTimer->isActive())
	{
		while(MousePollingMutex.tryLock());
		MousePollingMutex.unlock();
	
		MouseTimer->start(PollingTimeout);
	}
	//stepMouseTracking();
}


/*!
    \fn XEPlugin_Mouse::xEventDockerHidden()
 */
void XEPlugin_Mouse::xEventDockerHidden()
{
#ifndef ENABLE_FINAL
	qWarning("XEPlugin_Mouse::xEventDockerHidden()");
#endif
	if(!MouseTimer->isActive())
	{
		while(MousePollingMutex.tryLock());
		MousePollingMutex.unlock();
	
		MouseTimer->start(PollingTimeout);
	}
	//stepMouseTracking();
}


/*!
    \fn XEPlugin_Mouse::stepMouseTracking()
 */
void XEPlugin_Mouse::stepMouseTracking()
{
#ifndef ENABLE_FINAL
	qWarning("XEPlugin_Mouse::stepMouseTracking()");
#endif
	
	static int CountStep=0;
	static int PollingConversion=ActiveConfiguration->Window.SendToForgroundTimeout/PollingTimeout;

	// 0.22 support send to background which is not compatible with this!
	//if(xGDocker->getRaised()>0 || !xGDocker->isHidden())return;
	//0.27
	/*
	if(xGDocker->getRaised()>0 )return;
	if(MousePollingMutex.tryLock()==false)return;
	*/
	if(xGDocker->getRaised()>0 ||MousePollingMutex.tryLock()==false)
	{
		MouseTimer->stop();
		return;
	}
	
	
#ifndef ENABLE_FINAL
	qWarning("XEPlugin_Mouse::stepMouseTracking() 179");
#endif

	// 0.28
	// TODO: this work only on bottom
	XGIcon *borderIcons;
	int CoordsXIconsStop=ActiveConfiguration->Window.Width;
	int CoordsXIconsStart=0;
	// first icon
	borderIcons=xGDocker->getObjectIcon(0);
	if(borderIcons!=NULL)CoordsXIconsStart=borderIcons->xPosDefault.x()-(ActiveConfiguration->Icons.Size+ActiveConfiguration->Icons.Separation);
	// last icon
	borderIcons=xGDocker->getObjectIcon(ActiveConfiguration->ObjectsIcons.count()-1);
	if(borderIcons!=NULL)CoordsXIconsStop=borderIcons->xPosDefault.x()+(ActiveConfiguration->Icons.Size+ActiveConfiguration->Icons.Separation);
	
	
	
	QPoint pos, mouse, vect;
	int WinX, WinY;
	int XCoord,YCoord;
	XQueryPointer(display, root, &RootIDRet, &ChildIDRet, &XCoord,&YCoord, &WinX, &WinY, &StateMask);
	
	mouse = xGDocker->mapFromGlobal(QPoint(XCoord,YCoord));
	
#ifndef ENABLE_FINAL
	qWarning("XEPlugin_Mouse::stepMouseTracking() 190");
#endif
	if(strcmp(ActiveConfiguration->Window.Align,"bottom")==0)
	{
		// idea from georg
		// raise the bar only if there are 2 corner
		int MouseIsOk=0;
		
		if(ActiveConfiguration->Window.HideMouseCornerRight)
		{
		if(XCoord>=
		QApplication::desktop()->width()-
		ActiveConfiguration->Window.HideMouseEdge
		)
		{
			MouseIsOk++;
		}
		else MouseIsOk--;

		}
		if(ActiveConfiguration->Window.HideMouseCornerLeft)
		{
		if(XCoord<=
		ActiveConfiguration->Window.HideMouseEdge
		)
		{
			MouseIsOk++;
		}
		else MouseIsOk--;
		}
		if(
		(YCoord>=
		QApplication::desktop()->height()-
		ActiveConfiguration->Window.HideMouseEdge
		)
		&&
		(mouse.x()>=CoordsXIconsStart)
		&&
		(mouse.x()<=CoordsXIconsStop)
		// 0.28 patch enable only if you are over the icons
		)
		{
			MouseIsOk++;

		}
		else MouseIsOk--;

		if(MouseIsOk>0)
		{
				
			if(CountStep<PollingConversion)CountStep++;
			else
			{
			CountStep=0;
			// ok than we have to raise the widget!
			// 0.27
#ifndef ENABLE_FINAL
			qWarning("MouseTracking: xGDocker->showRaised();");
#endif
			MouseTimer->stop();
			xGDocker->showRaised();
			//xGDocker->show();
#ifndef ENABLE_FINAL
			qWarning("MouseTracking:End RAISED!!!");
#endif
			//MousePollingMutex.unlock();
			return;
			}

		}
		else CountStep=0;

	}
#ifndef ENABLE_FINAL
	qWarning("XEPlugin_Mouse::stepMouseTracking() 253");
#endif


	if(strcmp(ActiveConfiguration->Window.Align,"top")==0)
	{
		// idea from georg
		// raise the bar only if there are 2 corner
		int MouseIsOk=0;
		
		if(ActiveConfiguration->Window.HideMouseCornerRight)
		{
		if(XCoord>=
		QApplication::desktop()->width()-
		ActiveConfiguration->Window.HideMouseEdge
		)
		{
			MouseIsOk++;
		}
		else MouseIsOk--;
		
		}
		if(ActiveConfiguration->Window.HideMouseCornerLeft)
		{
		if(XCoord<=
		ActiveConfiguration->Window.HideMouseEdge
		)
		{
			MouseIsOk++;
		}
		else MouseIsOk--;
		
		}

		if(YCoord<=ActiveConfiguration->Window.HideMouseEdge
		)
		{
			MouseIsOk++;
		}
		else MouseIsOk--;

		if(MouseIsOk>0)
		{
			if(CountStep<PollingConversion)CountStep++;
			else
			{
			CountStep=0;
			// 0.27
#ifndef ENABLE_FINAL
			qWarning("MouseTracking: xGDocker->showRaised();");
#endif
			MouseTimer->stop();
			xGDocker->showRaised();
			//xGDocker->show();
#ifndef ENABLE_FINAL
			qWarning("MouseTracking:End RAISED!!!");
#endif
			//MousePollingMutex.unlock();
			return;
			}
		}
		else CountStep=0;

	}
#ifndef ENABLE_FINAL
	qWarning("MouseTracking:End 317");
#endif
	//QTimer::singleShot(PollingTimeout,this,SLOT(stepMouseTracking()));
	MousePollingMutex.unlock();
}


/*!
    \fn XEPlugin_Mouse::xGetParameter(const QString,QString&)
 */
void XEPlugin_Mouse::xGetParameter(const QString,QString&)
{
    /// @todo implement me
}


/*!
    \fn XEPlugin_Mouse::xGetParameterList(QStringList*)
 */
void XEPlugin_Mouse::xGetParameterList(QStringList*)
{
    /// @todo implement me
}
